@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2022, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2021, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -19,7 +19,6 @@
 #include "catalog/pg_class.h"
 #include "catalog/namespace.h"
 #include "commands/dbcommands.h"
-#include "commands/extension.h"
 #include "catalog/pg_proc.h"
 #include "commands/event_trigger.h"
 #include "executor/executor.h"
@@ -135,6 +134,21 @@ bool auditLogParameter = false;
  */
 bool auditLogRelation = false;
 
+/*
+ * GUC variable for pgaudit.log_rows
+ *
+ * Administrators can choose if the rows retrieved or affected by a statement
+ * are included in the audit log.
+ */
+bool auditLogRows = false;
+
+/*
+ * GUC variable for pgaudit.log_statement
+ *
+ * Administrators can choose to not have the full statement text logged.
+ */
+bool auditLogStatement = true;
+
 /*
  * GUC variable for pgaudit.log_statement_once
  *
@@ -219,6 +233,10 @@ typedef struct
     bool logged;                /* Track if we have logged this event, used
                                    post-ProcessUtility to make sure we log */
     bool statementLogged;       /* Track if we have logged the statement */
+    int64 rows;                 /* Track rows processed by the statement */
+    MemoryContext queryContext; /* Context for query tracking rows */
+    Oid auditOid;               /* Role running query tracking rows  */
+    List *rangeTabls;           /* Tables in query tracking rows */
 } AuditEvent;
 
 /*
@@ -397,6 +415,26 @@ stack_valid(int64 stackId)
              auditEventStack == NULL ? (int64) -1 : auditEventStack->stackId);
 }
 
+/*
+ * Find an item on the stack by the specified query memory context.
+ */
+static AuditEventStackItem *
+stack_find_context(MemoryContext findContext)
+{
+    AuditEventStackItem *nextItem = auditEventStack;
+
+    /* Look through the stack for the stack entry by query memory context */
+    while (nextItem != NULL)
+    {
+        if (nextItem->auditEvent.queryContext == findContext)
+            break;
+
+        nextItem = nextItem->next;
+    }
+
+    return nextItem;
+}
+
 /*
  * Appends a properly quoted CSV field to StringInfo.
  */
@@ -455,19 +493,6 @@ log_audit_event(AuditEventStackItem *stackItem)
     MemoryContext contextOld;
     StringInfoData auditStr;
 
-    /*
-     * Skip logging script statements if an extension is currently being created
-     * or altered. PostgreSQL reports the statement text for each statement in
-     * the script as the entire script text, which can blow up the logs. The
-     * create/alter statement will still be logged.
-     *
-     * Since a superuser is responsible for determining which extensions are
-     * available, and in most cases installing them, it should not be necessary
-     * to log each statement in the script.
-     */
-    if (creating_extension)
-        return;
-
     /* If this event has already been logged don't log it again */
     if (stackItem->auditEvent.logged)
         return;
@@ -669,7 +694,7 @@ log_audit_event(AuditEventStackItem *stackItem)
      * parameters if they have not already been logged for this substatement.
      */
     appendStringInfoCharMacro(&auditStr, ',');
-    if (!stackItem->auditEvent.statementLogged || !auditLogStatementOnce)
+    if (auditLogStatement && !(stackItem->auditEvent.statementLogged && auditLogStatementOnce))
     {
         append_valid_csv(&auditStr, stackItem->auditEvent.commandText);
 
@@ -723,11 +748,19 @@ log_audit_event(AuditEventStackItem *stackItem)
 
         stackItem->auditEvent.statementLogged = true;
     }
+    /* we were asked to not log it */
+    else if (!auditLogStatement)
+        appendStringInfoString(&auditStr,
+                               "<not logged>,<not logged>");
     else
-        /* we were asked to not log it */
         appendStringInfoString(&auditStr,
                                "<previously logged>,<previously logged>");
 
+    /* Log rows affected */
+    if (auditLogRows)
+        appendStringInfo(&auditStr, "," INT64_FORMAT,
+                         stackItem->auditEvent.rows);
+
     /*
      * Log the audit entry.  Note: use of INT64_FORMAT here is bad for
      * translatability, but we currently haven't got translation support in
@@ -1015,7 +1048,7 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         /*
          * We don't have access to the parsetree here, so we have to generate
          * the node type, object type, and command tag by decoding
-         * rte->requiredPerms and rte->relkind. For updates we also check
+         * rte->requiredPerms and rte->relkind. For updates we also check 
          * rellockmode so that only true UPDATE commands (not
          * SELECT FOR UPDATE, etc.) are logged as UPDATE.
          */
@@ -1239,6 +1272,9 @@ static ExecutorCheckPerms_hook_type next_ExecutorCheckPerms_hook = NULL;
 static ProcessUtility_hook_type next_ProcessUtility_hook = NULL;
 static object_access_hook_type next_object_access_hook = NULL;
 static ExecutorStart_hook_type next_ExecutorStart_hook = NULL;
+/* The following hook functions are required to get rows */
+static ExecutorRun_hook_type next_ExecutorRun_hook = NULL;
+static ExecutorEnd_hook_type next_ExecutorEnd_hook = NULL;
 
 /*
  * Hook ExecutorStart to get the query text and basic command type for queries
@@ -1308,8 +1344,14 @@ pgaudit_ExecutorStart_hook(QueryDesc *queryDesc, int eflags)
      * standard_ExecutorStart().
      */
     if (stackItem)
+    {
         MemoryContextSetParent(stackItem->contextAudit,
                                queryDesc->estate->es_query_cxt);
+
+        /* Set query context for tracking rows processed */
+        if (auditLogRows)
+            stackItem->auditEvent.queryContext = queryDesc->estate->es_query_cxt;
+    }
 }
 
 /*
@@ -1326,7 +1368,36 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, bool abort)
     /* Log DML if the audit role is valid or session logging is enabled */
     if ((auditOid != InvalidOid || auditLogBitmap != 0) &&
         !IsAbortedTransactionBlockState())
-        log_select_dml(auditOid, rangeTabls);
+    {
+        /* If auditLogRows is on, wait for rows processed to be set */
+        if (auditLogRows && auditEventStack != NULL)
+        {
+            /* Check if the top item is SELECT/INSERT for CREATE TABLE AS */
+            if (auditEventStack->auditEvent.commandTag == T_SelectStmt &&
+                auditEventStack->next != NULL &&
+                auditEventStack->next->auditEvent.command == CMDTAG_CREATE_TABLE_AS &&
+                auditEventStack->auditEvent.rangeTabls != NULL)
+            {
+                /*
+                 * First, log the INSERT event for CREATE TABLE AS here.
+                 * The SELECT event for CREATE TABLE AS will be logged
+                 * in pgaudit_ExecutorEnd_hook() later to get rows.
+                 */
+                log_select_dml(auditOid, rangeTabls);
+            }
+            else
+            {
+                /*
+                 * Save auditOid and rangeTabls to call log_select_dml()
+                 * in pgaudit_ExecutorEnd_hook() later.
+                 */
+                auditEventStack->auditEvent.auditOid = auditOid;
+                auditEventStack->auditEvent.rangeTabls = rangeTabls;
+            }
+        }
+        else
+            log_select_dml(auditOid, rangeTabls);
+    }
 
     /* Call the next hook function */
     if (next_ExecutorCheckPerms_hook &&
@@ -1336,12 +1407,74 @@ pgaudit_ExecutorCheckPerms_hook(List *rangeTabls, bool abort)
     return true;
 }
 
+/*
+ * Hook ExecutorRun to get rows processed by the current statement.
+ */
+static void
+pgaudit_ExecutorRun_hook(QueryDesc *queryDesc, ScanDirection direction, uint64 count, bool execute_once)
+{
+    AuditEventStackItem *stackItem = NULL;
+
+    /* Call the previous hook or standard function */
+    if (next_ExecutorRun_hook)
+        next_ExecutorRun_hook(queryDesc, direction, count, execute_once);
+    else
+        standard_ExecutorRun(queryDesc, direction, count, execute_once);
+
+    if (auditLogRows && !internalStatement)
+    {
+        /* Find an item from the stack by the query memory context */
+        stackItem = stack_find_context(queryDesc->estate->es_query_cxt);
+
+        /* Accumulate the number of rows processed */
+        if (stackItem != NULL)
+            stackItem->auditEvent.rows += queryDesc->estate->es_processed;
+    }
+}
+
+/*
+ * Hook ExecutorEnd to get rows processed by the current statement.
+ */
+static void
+pgaudit_ExecutorEnd_hook(QueryDesc *queryDesc)
+{
+    AuditEventStackItem *stackItem = NULL;
+    AuditEventStackItem *auditEventStackFull = NULL;
+
+    if (auditLogRows && !internalStatement)
+    {
+        /* Find an item from the stack by the query memory context */
+        stackItem = stack_find_context(queryDesc->estate->es_query_cxt);
+
+        if (stackItem != NULL && stackItem->auditEvent.rangeTabls != NULL)
+        {
+            /* Reset auditEventStack to use in log_select_dml() */
+            auditEventStackFull = auditEventStack;
+            auditEventStack = stackItem;
+
+            /* Log SELECT/DML audit entry */
+            log_select_dml(stackItem->auditEvent.auditOid,
+                           stackItem->auditEvent.rangeTabls);
+
+            /* Switch back to the previous auditEventStack */
+            auditEventStack = auditEventStackFull;
+        }
+    }
+
+    /* Call the previous hook or standard function */
+    if (next_ExecutorEnd_hook)
+        next_ExecutorEnd_hook(queryDesc);
+    else
+        standard_ExecutorEnd(queryDesc);
+}
+
 /*
  * Hook ProcessUtility to do session auditing for DDL and utility commands.
  */
 static void
 pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
                             const char *queryString,
+                            bool readOnlyTree,
                             ProcessUtilityContext context,
                             ParamListInfo params,
                             QueryEnvironment *queryEnv,
@@ -1403,18 +1536,6 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
 
-        /*
-         * If this is a create/alter extension command log it before calling
-         * the next ProcessUtility hook. Otherwise, any warnings will be emitted
-         * before the create/alter is logged and errors will prevent it from
-         * being logged at all.
-         */
-        if (auditLogBitmap & LOG_DDL &&
-            (stackItem->auditEvent.commandTag == T_CreateExtensionStmt ||
-                stackItem->auditEvent.commandTag == T_AlterExtensionStmt) &&
-            !IsAbortedTransactionBlockState())
-            log_audit_event(stackItem);
-
         /*
          * A close will free the open cursor which will also free the close
          * audit entry. Immediately log the close and set stackItem to NULL so
@@ -1431,11 +1552,11 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
 
     /* Call the standard process utility chain. */
     if (next_ProcessUtility_hook)
-        (*next_ProcessUtility_hook) (pstmt, queryString, context, params,
-                                     queryEnv, dest, qc);
+        (*next_ProcessUtility_hook) (pstmt, queryString, readOnlyTree, context,
+                                     params, queryEnv, dest, qc);
     else
-        standard_ProcessUtility(pstmt, queryString, context, params,
-                                queryEnv, dest, qc);
+        standard_ProcessUtility(pstmt, queryString, readOnlyTree, context,
+                                params, queryEnv, dest, qc);
 
     /*
      * Process the audit event if there is one.  Also check that this event
@@ -1963,6 +2084,35 @@ _PG_init(void)
         GUC_NOT_IN_SAMPLE,
         NULL, NULL, NULL);
 
+    /* Define pgaudit.log_rows */
+    DefineCustomBoolVariable(
+        "pgaudit.log_rows",
+
+        "Specifies whether logging will include the rows retrieved or "
+        "affected by a statement.",
+
+        NULL,
+        &auditLogRows,
+        false,
+        PGC_SUSET,
+        GUC_NOT_IN_SAMPLE,
+        NULL, NULL, NULL);
+
+    /* Define pgaudit.log_statement */
+    DefineCustomBoolVariable(
+        "pgaudit.log_statement",
+
+        "Specifies whether logging will include the statement text and "
+        "parameters.  Depending on requirements, the full statement text might "
+        "not be required in the audit log.",
+
+        NULL,
+        &auditLogStatement,
+        true,
+        PGC_SUSET,
+        GUC_NOT_IN_SAMPLE,
+        NULL, NULL, NULL);
+
     /* Define pgaudit.log_statement_once */
     DefineCustomBoolVariable(
         "pgaudit.log_statement_once",
@@ -2014,6 +2164,13 @@ _PG_init(void)
     next_object_access_hook = object_access_hook;
     object_access_hook = pgaudit_object_access_hook;
 
+    /* The following hook functions are required to get rows */
+    next_ExecutorRun_hook = ExecutorRun_hook;
+    ExecutorRun_hook = pgaudit_ExecutorRun_hook;
+
+    next_ExecutorEnd_hook = ExecutorEnd_hook;
+    ExecutorEnd_hook = pgaudit_ExecutorEnd_hook;
+
     /* Log that the extension has completed initialization */
 #ifndef EXEC_BACKEND
     ereport(LOG, (errmsg("pgaudit extension initialized")));
