@@ -3,9 +3,6 @@
 -- Create pgaudit extension
 CREATE EXTENSION IF NOT EXISTS pgaudit;
 
--- Grant all on public schema to public
-GRANT ALL ON SCHEMA public TO public;
-
 -- Make sure events don't get logged twice when session logging
 SET pgaudit.log = 'all';
 SET pgaudit.log_client = ON;
@@ -35,6 +32,7 @@ RESET pgaudit.log_level;
 --     STATEMENT - The statement being logged
 --     PARAMETER - If parameter logging is requested, they will follow the
 --                 statement
+--     ROWS - If rows logging is requested, they will follow the parameter
 
 SELECT current_user \gset
 
@@ -48,23 +46,23 @@ ALTER ROLE :current_user SET pgaudit.log_client = ON;
 
 --
 -- Create auditor role
-CREATE ROLE regress_auditor;
+CREATE ROLE auditor;
 
 --
 -- Create first test user
-CREATE USER regress_user1 password 'password';
-ALTER ROLE regress_user1 SET pgaudit.log = 'ddl, ROLE';
-ALTER ROLE regress_user1 SET pgaudit.log_level = 'notice';
+CREATE USER user1 password 'password';
+ALTER ROLE user1 SET pgaudit.log = 'ddl, ROLE';
+ALTER ROLE user1 SET pgaudit.log_level = 'notice';
 
-ALTER ROLE regress_user1 PassWord 'password2' NOLOGIN;
-ALTER USER regress_user1 encrypted /* random comment */PASSWORD
+ALTER ROLE user1 PassWord 'password2' NOLOGIN;
+ALTER USER user1 encrypted /* random comment */PASSWORD
 	/* random comment */
     'md565cb1da342495ea6bb0418a6e5718c38' LOGIN;
-ALTER ROLE regress_user1 SET pgaudit.log_client = ON;
+ALTER ROLE user1 SET pgaudit.log_client = ON;
 
 --
 -- Create, select, drop (select will not be audited)
-\connect - regress_user1
+\connect - user1
 
 CREATE TABLE public.test
 (
@@ -80,13 +78,13 @@ DROP TABLE test;
 -- Create second test user
 \connect - :current_user
 
-CREATE ROLE regress_user2 LOGIN password 'password';
-ALTER ROLE regress_user2 SET pgaudit.log = 'Read, writE';
-ALTER ROLE regress_user2 SET pgaudit.log_catalog = OFF;
-ALTER ROLE regress_user2 SET pgaudit.log_client = ON;
-ALTER ROLE regress_user2 SET pgaudit.log_level = 'warning';
-ALTER ROLE regress_user2 SET pgaudit.role = regress_auditor;
-ALTER ROLE regress_user2 SET pgaudit.log_statement_once = ON;
+CREATE ROLE user2 LOGIN password 'password';
+ALTER ROLE user2 SET pgaudit.log = 'Read, writE';
+ALTER ROLE user2 SET pgaudit.log_catalog = OFF;
+ALTER ROLE user2 SET pgaudit.log_client = ON;
+ALTER ROLE user2 SET pgaudit.log_level = 'warning';
+ALTER ROLE user2 SET pgaudit.role = auditor;
+ALTER ROLE user2 SET pgaudit.log_statement_once = ON;
 
 --
 -- Setup role-based tests
@@ -97,11 +95,11 @@ CREATE TABLE test2
 
 GRANT SELECT, INSERT, UPDATE, DELETE
    ON test2
-   TO regress_user2, regress_user1;
+   TO user2, user1;
 
 GRANT SELECT, UPDATE
    ON TABLE public.test2
-   TO regress_auditor;
+   TO auditor;
 
 CREATE TABLE test3
 (
@@ -110,11 +108,11 @@ CREATE TABLE test3
 
 GRANT SELECT, INSERT, UPDATE, DELETE
    ON test3
-   TO regress_user2;
+   TO user2;
 
 GRANT INSERT
    ON TABLE public.test3
-   TO regress_auditor;
+   TO auditor;
 
 CREATE FUNCTION test2_insert() RETURNS TRIGGER AS $$
 BEGIN
@@ -124,7 +122,7 @@ BEGIN
 
 	RETURN new;
 END $$ LANGUAGE plpgsql security definer;
-ALTER FUNCTION test2_insert() OWNER TO regress_user1;
+ALTER FUNCTION test2_insert() OWNER TO user1;
 
 CREATE TRIGGER test2_insert_trg
 	AFTER INSERT ON test2
@@ -136,7 +134,7 @@ BEGIN
 	   SET id = id + 1
 	 WHERE id = change_id;
 END $$ LANGUAGE plpgsql security definer;
-ALTER FUNCTION test2_change(int) OWNER TO regress_user2;
+ALTER FUNCTION test2_change(int) OWNER TO user2;
 
 CREATE VIEW vw_test3 AS
 SELECT *
@@ -144,13 +142,13 @@ SELECT *
 
 GRANT SELECT
    ON vw_test3
-   TO regress_user2;
+   TO user2;
 
 GRANT SELECT
    ON vw_test3
-   TO regress_auditor;
+   TO auditor;
 
-\connect - regress_user2
+\connect - user2
 
 --
 -- Role-based tests
@@ -236,9 +234,9 @@ SELECT *
 --
 -- Change permissions of user 2 so that only object logging will be done
 \connect - :current_user
-ALTER ROLE regress_user2 SET pgaudit.log = 'NONE';
+ALTER ROLE user2 SET pgaudit.log = 'NONE';
 
-\connect - regress_user2
+\connect - user2
 
 --
 -- Create test4 and add permissions
@@ -250,15 +248,15 @@ CREATE TABLE test4
 
 GRANT SELECT (name)
    ON TABLE public.test4
-   TO regress_auditor;
+   TO auditor;
 
 GRANT UPDATE (id)
    ON TABLE public.test4
-   TO regress_auditor;
+   TO auditor;
 
 GRANT insert (name)
    ON TABLE public.test4
-   TO regress_auditor;
+   TO auditor;
 
 --
 -- Not object logged
@@ -312,8 +310,8 @@ DROP TABLE test4;
 DROP FUNCTION test2_insert();
 DROP FUNCTION test2_change(int);
 
-ALTER ROLE regress_user1 SET pgaudit.log = 'DDL, READ';
-\connect - regress_user1
+ALTER ROLE user1 SET pgaudit.log = 'DDL, READ';
+\connect - user1
 
 --
 -- Create table is session logged
@@ -333,21 +331,21 @@ SELECT *
 --
 -- Insert is not logged
 INSERT INTO account (id, name, password, description)
-			 VALUES (1, 'regress_user1', 'HASH1', 'blah, blah');
+			 VALUES (1, 'user1', 'HASH1', 'blah, blah');
 
 --
 -- Change permissions of user 1 so that only object logging will be done
 \connect - :current_user
-ALTER ROLE regress_user1 SET pgaudit.log = 'none';
-ALTER ROLE regress_user1 SET pgaudit.role = 'regress_auditor';
-\connect - regress_user1
+ALTER ROLE user1 SET pgaudit.log = 'none';
+ALTER ROLE user1 SET pgaudit.role = 'auditor';
+\connect - user1
 
 --
--- ROLE class not set, so regress_auditor grants not logged
+-- ROLE class not set, so auditor grants not logged
 GRANT SELECT (password),
 	  UPDATE (name, password)
    ON TABLE public.account
-   TO regress_auditor;
+   TO auditor;
 
 --
 -- Not object logged
@@ -375,9 +373,9 @@ UPDATE account
 --
 -- Change permissions of user 1 so that session relation logging will be done
 \connect - :current_user
-ALTER ROLE regress_user1 SET pgaudit.log_relation = on;
-ALTER ROLE regress_user1 SET pgaudit.log = 'read, WRITE';
-\connect - regress_user1
+ALTER ROLE user1 SET pgaudit.log_relation = on;
+ALTER ROLE user1 SET pgaudit.log = 'read, WRITE';
+\connect - user1
 
 --
 -- Not logged
@@ -388,10 +386,10 @@ CREATE TABLE ACCOUNT_ROLE_MAP
 );
 
 --
--- ROLE class not set, so regress_auditor grants not logged
+-- ROLE class not set, so auditor grants not logged
 GRANT SELECT
    ON TABLE public.account_role_map
-   TO regress_auditor;
+   TO auditor;
 
 --
 -- Object logged because of:
@@ -441,6 +439,18 @@ UPDATE account
 UPDATE account
    SET password = 'HASH2';
 
+--
+-- Change configuration of user 1 so that full statements are not logged
+\connect - :current_user
+ALTER ROLE user1 RESET pgaudit.log_relation;
+ALTER ROLE user1 RESET pgaudit.log;
+ALTER ROLE user1 SET pgaudit.log_statement = OFF;
+\connect - user1
+
+--
+-- Logged but without full statement
+SELECT * FROM account;
+
 --
 -- Change back to superuser to do exhaustive tests
 \connect - :current_user
@@ -474,7 +484,7 @@ SELECT *
 --
 -- Copy from stdin to account copy
 COPY test.account_copy from stdin;
-1	regress_user1	HASH2	yada, yada
+1	user1	HASH2	yada, yada
 \.
 
 --
@@ -668,15 +678,15 @@ DROP DATABASE contrib_regression_pgaudit2;
 SET pgaudit.log = 'ROLE';
 
 CREATE TABLE t ();
-CREATE ROLE regress_alice;
+CREATE ROLE alice;
 
 CREATE SCHEMA foo2
 	GRANT SELECT
 	   ON public.t
-	   TO regress_alice;
+	   TO alice;
 
 drop table public.t;
-drop role regress_alice;
+drop role alice;
 
 --
 -- Test for non-empty stack error
@@ -722,7 +732,7 @@ SELECT test();
 --
 -- Delete all rows then delete 1 row
 SET pgaudit.log = 'write';
-SET pgaudit.role = 'regress_auditor';
+SET pgaudit.role = 'auditor';
 
 create table bar
 (
@@ -731,7 +741,7 @@ create table bar
 
 grant delete
    on bar
-   to regress_auditor;
+   to auditor;
 
 insert into bar (col)
 		 values (1);
@@ -747,13 +757,13 @@ drop table bar;
 --
 -- Grant roles to each other
 SET pgaudit.log = 'role';
-GRANT regress_user1 TO regress_user2;
-REVOKE regress_user1 FROM regress_user2;
+GRANT user1 TO user2;
+REVOKE user1 FROM user2;
 
 --
 -- Test that FK references do not log but triggers still do
 SET pgaudit.log = 'READ,WRITE';
-SET pgaudit.role TO 'regress_auditor';
+SET pgaudit.role TO 'auditor';
 
 CREATE TABLE aaa
 (
@@ -779,11 +789,11 @@ CREATE TRIGGER bbb_insert_trg
 
 GRANT SELECT, UPDATE
    ON aaa
-   TO regress_auditor;
+   TO auditor;
 
 GRANT UPDATE
    ON bbb
-   TO regress_auditor;
+   TO auditor;
 
 INSERT INTO aaa VALUES (generate_series(1,100));
 
@@ -811,9 +821,9 @@ SET pgaudit.log_client = ON;
 SET pgaudit.log_relation = ON;
 SET pgaudit.log_parameter = ON;
 
-CREATE ROLE regress_alice;
+CREATE ROLE alice;
 
-SET ROLE regress_alice;
+SET ROLE alice;
 CREATE TABLE t (a int, b text);
 SET search_path TO test, public;
 
@@ -826,7 +836,7 @@ RESET ROLE;
 -- Test MISC_SET
 SET pgaudit.log = 'MISC_SET';
 
-SET ROLE regress_alice;
+SET ROLE alice;
 SET search_path TO public;
 
 INSERT INTO t VALUES (2, 'misc_set');
@@ -846,7 +856,7 @@ VACUUM t;
 
 RESET ROLE;
 DROP TABLE public.t;
-DROP ROLE regress_alice;
+DROP ROLE alice;
 
 --
 -- Test PARTITIONED table
@@ -860,6 +870,681 @@ CREATE INDEX h_idx ON h (x);
 DROP INDEX h_idx;
 DROP TABLE h;
 
+--
+-- Test rows retrived or affected by statements
+\connect - :current_user
+
+SET pgaudit.log = 'all';
+SET pgaudit.log_client = on;
+SET pgaudit.log_relation = on;
+SET pgaudit.log_statement_once = off;
+SET pgaudit.log_parameter = on;
+SET pgaudit.log_rows = on;
+
+--
+-- Test DDL
+CREATE TABLE test2
+(
+	id int,
+	name text
+);
+
+CREATE TABLE test3
+(
+	id int,
+	name text
+);
+
+CREATE FUNCTION test2_insert() RETURNS TRIGGER AS $$
+BEGIN
+	UPDATE test2
+	   SET id = id + 90
+	 WHERE id = new.id;
+
+	RETURN new;
+END $$ LANGUAGE plpgsql security definer;
+
+CREATE TRIGGER test2_insert_trg
+	AFTER INSERT ON test2
+	FOR EACH ROW EXECUTE PROCEDURE test2_insert();
+
+CREATE FUNCTION test2_change(change_id int) RETURNS void AS $$
+BEGIN
+	UPDATE test2
+	   SET id = id + 1
+	 WHERE id = change_id;
+END $$ LANGUAGE plpgsql security definer;
+
+CREATE VIEW vw_test3 AS
+SELECT *
+  FROM test3;
+
+--
+-- Test DML
+INSERT INTO test2 (id, name)
+		  VALUES (1, 'a');
+INSERT INTO test2 (id, name)
+		  VALUES (2, 'b');
+INSERT INTO test2 (id, name)
+		  VALUES (3, 'c');
+
+INSERT INTO test3 (id, name)
+		  VALUES (1, 'a');
+INSERT INTO test3 (id, name)
+		  VALUES (2, 'b');
+INSERT INTO test3 (id, name)
+		  VALUES (3, 'c');
+
+SELECT *
+  FROM test3, test2;
+
+SELECT *
+  FROM vw_test3, test2;
+
+SELECT *
+  FROM test2
+ ORDER BY ID;
+
+UPDATE test2
+   SET name = 'd';
+
+UPDATE test3
+   SET name = 'd'
+ WHERE id > 0;
+
+SELECT 1
+  FROM
+(
+	SELECT relname
+	  FROM pg_class
+	  LIMIT 3
+) SUBQUERY;
+
+WITH CTE AS
+(
+	SELECT id
+	  FROM test2
+)
+INSERT INTO test3
+SELECT id
+  FROM cte;
+
+WITH CTE AS
+(
+	INSERT INTO test3 VALUES (1)
+				   RETURNING id
+)
+INSERT INTO test2
+SELECT id
+  FROM cte;
+
+DO $$ BEGIN PERFORM test2_change(91); END $$;
+
+WITH CTE AS
+(
+	UPDATE test2
+	   SET id = 45
+	 WHERE id = 92
+	RETURNING id
+)
+INSERT INTO test3
+SELECT id
+  FROM cte;
+
+WITH CTE AS
+(
+	INSERT INTO test2 VALUES (37)
+				   RETURNING id
+)
+UPDATE test3
+   SET id = cte.id
+  FROM cte
+ WHERE test3.id <> cte.id;
+
+DELETE
+  FROM test2;
+
+DELETE
+  FROM test3
+ WHERE id > 0;
+
+--
+-- Drop test tables
+DROP TABLE test2;
+DROP VIEW vw_test3;
+DROP TABLE test3;
+DROP FUNCTION test2_insert();
+DROP FUNCTION test2_change(int);
+
+--
+-- Only object logging will be done
+SET pgaudit.log = 'none';
+SET pgaudit.role = 'auditor';
+
+--
+-- Select is session logged
+SELECT *
+  FROM account;
+
+--
+-- Insert is not logged
+INSERT INTO account (id, name, password, description)
+			 VALUES (1, 'user2', 'HASH3', 'blah, blah2');
+INSERT INTO account (id, name, password, description)
+			 VALUES (1, 'user3', 'HASH4', 'blah, blah3');
+
+--
+-- Not object logged
+SELECT id,
+	   name
+  FROM account;
+
+--
+-- Object logged because of:
+-- select (password) on account
+SELECT password
+  FROM account;
+
+--
+-- Not object logged
+UPDATE account
+   SET description = 'yada, yada1';
+
+--
+-- Object logged because of:
+-- update (password) on account
+UPDATE account
+   SET password = 'HASH4';
+
+--
+-- Session relation logging will be done
+SET pgaudit.log_relation = on;
+SET pgaudit.log = 'read, write';
+
+--
+-- Object logged because of:
+-- select (password) on account
+-- select on account_role_map
+-- Session logged on all tables because log = read and log_relation = on
+SELECT account.password,
+	   account_role_map.role_id
+  FROM account
+	   INNER JOIN account_role_map
+			on account.id = account_role_map.account_id;
+
+--
+-- Object logged because of:
+-- select (password) on account
+-- Session logged on all tables because log = read and log_relation = on
+SELECT password
+  FROM account;
+
+--
+-- Not object logged
+-- Session logged on all tables because log = read and log_relation = on
+UPDATE account
+   SET description = 'yada, yada2';
+
+--
+-- Object logged because of:
+-- select (password) on account (in the where clause)
+-- Session logged on all tables because log = read and log_relation = on
+UPDATE account
+   SET description = 'yada, yada3'
+ where password = 'HASH4';
+
+--
+-- Object logged because of:
+-- update (password) on account
+-- Session logged on all tables because log = read and log_relation = on
+UPDATE account
+   SET password = 'HASH4';
+
+--
+-- Exhaustive tests
+SET pgaudit.log = 'all';
+SET pgaudit.log_client = on;
+SET pgaudit.log_relation = on;
+SET pgaudit.log_parameter = on;
+SET pgaudit.log_rows = on;
+
+--
+-- Simple DO block
+DO $$
+BEGIN
+	raise notice 'test';
+END $$;
+
+--
+-- Copy account to stdout
+COPY account TO stdout;
+
+--
+-- Drop a table from a query
+DROP TABLE test.account_copy;
+
+--
+-- Create a table from a query
+CREATE TABLE test.account_copy AS
+SELECT *
+  FROM account;
+
+--
+-- Copy from stdin to account copy
+COPY test.account_copy from stdin;
+1	user1	HASH4	yada, yada4
+\.
+
+--
+-- Test prepared statement
+PREPARE pgclassstmt1 (oid) AS
+SELECT 1
+  FROM account
+ WHERE id = $1;
+
+PREPARE pgclassstmt2 (oid) AS
+SELECT 2
+  FROM account
+ WHERE id = $1;
+
+EXECUTE pgclassstmt2 (1);
+EXECUTE pgclassstmt1 (1);
+
+DEALLOCATE pgclassstmt2;
+DEALLOCATE pgclassstmt1;
+
+--
+-- Test cursor
+BEGIN;
+
+DECLARE ctest1 SCROLL CURSOR FOR
+SELECT 1
+  FROM
+(
+	SELECT relname
+	  FROM pg_class
+	 LIMIT 3
+ ) subquery;
+
+DECLARE ctest2 SCROLL CURSOR FOR
+SELECT 2
+  FROM
+(
+	SELECT relname
+	  FROM pg_class
+	 LIMIT 3
+ ) subquery;
+
+FETCH NEXT FROM ctest1;
+FETCH NEXT FROM ctest2;
+FETCH NEXT FROM ctest1;
+FETCH NEXT FROM ctest2;
+FETCH NEXT FROM ctest1;
+FETCH NEXT FROM ctest2;
+
+CLOSE ctest2;
+CLOSE ctest1;
+COMMIT;
+
+--
+-- Turn off log_catalog and pg_class will not be logged
+SET pgaudit.log_catalog = off;
+
+SELECT count(*)
+  FROM
+(
+	SELECT relname
+	  FROM pg_class
+	 LIMIT 1
+ ) subquery;
+
+--
+-- Test prepared insert
+DROP TABLE test.test_insert;
+
+--
+-- Test prepared insert
+CREATE TABLE test.test_insert
+(
+	id INT
+);
+
+PREPARE pgclassstmt (oid) AS
+INSERT INTO test.test_insert (id)
+					  VALUES ($1);
+EXECUTE pgclassstmt (1);
+
+--
+-- Check that primary key creation is logged
+CREATE TABLE public.test
+(
+	id INT,
+	name TEXT,
+	description TEXT,
+	CONSTRAINT test_pkey PRIMARY KEY (id)
+);
+
+--
+-- Check that analyze is logged
+ANALYZE test;
+
+--
+-- Grants to public should not cause object logging (session logging will
+-- still happen)
+GRANT SELECT
+  ON TABLE public.test
+  TO PUBLIC;
+
+SELECT *
+  FROM test;
+
+-- Check that statements without columns log
+SELECT
+  FROM test;
+
+SELECT 1,
+	   substring('Thomas' from 2 for 3);
+
+DO $$
+DECLARE
+	test INT;
+BEGIN
+	SELECT 1
+	  INTO test;
+END $$;
+
+explain select 1;
+
+--
+-- Test that looks inside of do blocks log
+INSERT INTO TEST (id)
+		  VALUES (1);
+INSERT INTO TEST (id)
+		  VALUES (2);
+INSERT INTO TEST (id)
+		  VALUES (3);
+
+DO $$
+DECLARE
+	result RECORD;
+BEGIN
+	FOR result IN
+		SELECT id
+		  FROM test
+	LOOP
+		INSERT INTO test (id)
+			 VALUES (result.id + 100);
+	END LOOP;
+END $$;
+
+--
+-- Test obfuscated dynamic sql for clean logging
+DO $$
+DECLARE
+	table_name TEXT = 'do_table';
+BEGIN
+	EXECUTE 'CREATE TABLE ' || table_name || ' ("weird name" INT)';
+	EXECUTE 'DROP table ' || table_name;
+END $$;
+
+--
+-- Generate an error and make sure the stack gets cleared
+DO $$
+BEGIN
+	CREATE TABLE bogus.test_block
+	(
+		id INT
+	);
+END $$;
+
+--
+-- Test alter table statements
+ALTER TABLE public.test
+	DROP COLUMN description ;
+
+ALTER TABLE public.test
+	RENAME TO test2;
+
+ALTER TABLE public.test2
+	SET SCHEMA test;
+
+ALTER TABLE test.test2
+	ADD COLUMN description TEXT;
+
+ALTER TABLE test.test2
+	DROP COLUMN description;
+
+DROP TABLE test.test2;
+
+--
+-- Test multiple statements with one semi-colon
+CREATE SCHEMA foo1
+	CREATE TABLE foo1.bar1 (id int)
+	CREATE TABLE foo1.baz1 (id int);
+
+DROP TABLE foo1.bar1;
+DROP TABLE foo1.baz1;
+DROP SCHEMA foo1;
+
+--
+-- Test aggregate
+CREATE FUNCTION public.int_add1
+(
+	a INT,
+	b INT
+)
+	RETURNS INT LANGUAGE plpgsql AS $$
+BEGIN
+	return a + b;
+END $$;
+
+SELECT int_add1(1, 1);
+
+CREATE AGGREGATE public.sum_test(INT) (SFUNC=public.int_add1, STYPE=INT, INITCOND='0');
+ALTER AGGREGATE public.sum_test(integer) RENAME TO sum_test3;
+
+--
+-- Test conversion
+CREATE CONVERSION public.conversion_test FOR 'latin1' TO 'utf8' FROM pg_catalog.iso8859_1_to_utf8;
+ALTER CONVERSION public.conversion_test RENAME TO conversion_test3;
+
+--
+-- Test create/alter/drop database
+CREATE DATABASE contrib_regression_pgaudit;
+ALTER DATABASE contrib_regression_pgaudit RENAME TO contrib_regression_pgaudit2;
+DROP DATABASE contrib_regression_pgaudit2;
+
+-- Test role as a substmt
+SET pgaudit.log = 'role';
+
+CREATE TABLE t ();
+CREATE ROLE alice;
+
+CREATE SCHEMA foo3
+	GRANT SELECT
+	   ON public.t
+	   TO alice;
+
+drop table public.t;
+drop role alice;
+
+--
+-- Test for non-empty stack error
+CREATE OR REPLACE FUNCTION get_test_id(_ret REFCURSOR) RETURNS REFCURSOR
+LANGUAGE plpgsql IMMUTABLE AS $$
+BEGIN
+    OPEN _ret FOR SELECT 200;
+    RETURN _ret;
+END $$;
+
+BEGIN;
+    SELECT get_test_id('_ret');
+    SELECT get_test_id('_ret2');
+    FETCH ALL FROM _ret;
+    FETCH ALL FROM _ret2;
+    CLOSE _ret;
+    CLOSE _ret2;
+END;
+
+--
+-- Test that frees a memory context earlier than expected
+SET pgaudit.log = 'all';
+
+CREATE FUNCTION test1()
+	RETURNS INT AS $$
+DECLARE
+	cur1 cursor for select * from hoge;
+	tmp int;
+BEGIN
+	OPEN cur1;
+	FETCH cur1 into tmp;
+	RETURN tmp;
+END $$
+LANGUAGE plpgsql ;
+
+SELECT test1();
+
+--
+-- Delete all rows then delete 1 row
+SET pgaudit.log = 'write';
+SET pgaudit.role = 'auditor';
+
+create table bar
+(
+	col int
+);
+
+grant delete
+   on bar
+   to auditor;
+
+insert into bar (col)
+		 values (1);
+delete from bar;
+
+insert into bar (col)
+		 values (1);
+delete from bar
+ where col = 1;
+
+drop table bar;
+
+--
+-- Test that FK references do not log but triggers still do
+SET pgaudit.log = 'read, write';
+SET pgaudit.role TO 'auditor';
+
+CREATE TABLE aaa
+(
+	ID int primary key
+);
+
+CREATE TABLE bbb
+(
+	id int
+		references aaa(id)
+);
+
+CREATE FUNCTION bbb_insert1() RETURNS TRIGGER AS $$
+BEGIN
+	UPDATE bbb set id = new.id + 1;
+
+	RETURN new;
+END $$ LANGUAGE plpgsql;
+
+CREATE TRIGGER bbb_insert_trg1
+	AFTER INSERT ON bbb
+	FOR EACH ROW EXECUTE PROCEDURE bbb_insert1();
+
+GRANT SELECT, UPDATE
+   ON aaa
+   TO auditor;
+
+GRANT UPDATE
+   ON bbb
+   TO auditor;
+
+INSERT INTO aaa VALUES (generate_series(1,100));
+
+SET pgaudit.log_parameter TO OFF;
+INSERT INTO bbb VALUES (1);
+SET pgaudit.log_parameter TO ON;
+
+DROP TABLE bbb;
+DROP TABLE aaa;
+
+--
+-- Test MISC
+SET pgaudit.log = 'misc';
+SET pgaudit.log_client = on;
+SET pgaudit.log_relation = on;
+SET pgaudit.log_parameter = on;
+
+CREATE ROLE alice;
+
+SET ROLE alice;
+CREATE TABLE t (a int, b text);
+SET search_path TO test, public;
+
+INSERT INTO t VALUES (1, 'misc');
+
+VACUUM t;
+RESET ROLE;
+
+--
+-- Test MISC_SET
+SET pgaudit.log = 'MISC_SET';
+
+SET ROLE alice;
+SET search_path TO public;
+
+INSERT INTO t VALUES (2, 'misc_set');
+
+VACUUM t;
+RESET ROLE;
+
+--
+-- Test ALL, -MISC, MISC_SET
+SET pgaudit.log = 'ALL, -MISC, MISC_SET';
+SET search_path TO public;
+
+INSERT INTO t VALUES (3, 'all, -misc, misc_set');
+
+VACUUM t;
+
+RESET ROLE;
+DROP TABLE public.t;
+DROP ROLE alice;
+
+--
+-- Test PARTITIONED table
+CREATE TABLE h(x int ,y int) PARTITION BY HASH(x);
+CREATE TABLE h_0 partition OF h FOR VALUES WITH ( MODULUS 2, REMAINDER 0);
+CREATE TABLE h_1 partition OF h FOR VALUES WITH ( MODULUS 2, REMAINDER 1);
+INSERT INTO h VALUES(1,1);
+SELECT * FROM h;
+SELECT * FROM h_0;
+CREATE INDEX h_idx ON h (x);
+DROP INDEX h_idx;
+DROP TABLE h;
+
+--
+-- Change configuration of user 1 so that full statements are not logged
+\connect - :current_user
+ALTER ROLE user1 RESET pgaudit.log_relation;
+ALTER ROLE user1 RESET pgaudit.log;
+ALTER ROLE user1 SET pgaudit.log_statement = OFF;
+ALTER ROLE user1 SET pgaudit.log_rows = on;
+\connect - user1
+
+--
+-- Logged but without full statement
+SELECT * FROM account;
+
+--
+-- Change back to superuser to do exhaustive tests
+\connect - :current_user
+
 --
 -- Test that pgaudit event triggers are immune to search-path-based attacks
 
@@ -892,38 +1577,6 @@ DROP FUNCTION my_ne(text, text);
 DROP FUNCTION lower(text);
 DROP FUNCTION upper(text);
 
--- Create/drop extension. Note that the log level here must be warning because the create extension code will reset any higher log
--- levels like notice
-SET pgaudit.log = 'all,-misc_set';
-SET pgaudit.log_level = 'warning';
-
-CREATE EXTENSION pg_stat_statements;
-ALTER EXTENSION pg_stat_statements UPDATE TO '1.8';
-DROP EXTENSION pg_stat_statements;
-
-SET pgaudit.log_level = 'notice';
-
--- Check that partition scans are skipped for auditing and do no result in an empty stack
-SET pgaudit.log = 'all';
-
-CREATE TABLE part_test (c1 int, c2 int) PARTITION BY RANGE (c1);
-CREATE TABLE part_test_1_to_10 PARTITION OF part_test FOR VALUES FROM (1) TO (10);
-INSERT INTO part_test VALUES (generate_series(1,9));
-
-CREATE OR REPLACE FUNCTION get_test_id(_ret REFCURSOR) RETURNS REFCURSOR
-LANGUAGE plpgsql IMMUTABLE AS $$
-BEGIN
-    OPEN _ret FOR SELECT * FROM part_test;
-    RETURN _ret;
-END $$;
-
-BEGIN;
-SELECT get_test_id('_ret');
-FETCH ALL FROM _ret;
-COMMIT;
-
-DROP TABLE part_test;
-
 -- Cleanup
 -- Set client_min_messages up to warning to avoid noise
 SET client_min_messages = 'warning';
@@ -934,6 +1587,7 @@ ALTER ROLE :current_user RESET pgaudit.log_client;
 ALTER ROLE :current_user RESET pgaudit.log_level;
 ALTER ROLE :current_user RESET pgaudit.log_parameter;
 ALTER ROLE :current_user RESET pgaudit.log_relation;
+ALTER ROLE :current_user RESET pgaudit.log_statement;
 ALTER ROLE :current_user RESET pgaudit.log_statement_once;
 ALTER ROLE :current_user RESET pgaudit.role;
 
@@ -942,6 +1596,7 @@ RESET pgaudit.log_catalog;
 RESET pgaudit.log_level;
 RESET pgaudit.log_parameter;
 RESET pgaudit.log_relation;
+RESET pgaudit.log_statement;
 RESET pgaudit.log_statement_once;
 RESET pgaudit.role;
 
@@ -954,8 +1609,8 @@ DROP SCHEMA foo;
 DROP TABLE hoge;
 DROP TABLE account;
 DROP TABLE account_role_map;
-DROP USER regress_user2;
-DROP USER regress_user1;
-DROP ROLE regress_auditor;
+DROP USER user2;
+DROP USER user1;
+DROP ROLE auditor;
 
 RESET client_min_messages;
